@@ -944,7 +944,7 @@ on_build_terminated (AnjutaLauncher *launcher,
 	if (context->program->callback != NULL)
 	{
 		GError *err = NULL;
-		
+
 		if (WIFEXITED (status))
 		{
 			if (WEXITSTATUS (status) != 0)
@@ -1257,7 +1257,7 @@ build_execute_command_in_context (BuildContext* context, GError **err)
 		build_context_destroy_command (context);
 		return FALSE;
 	}
-	
+
 	if (context->message_view)
 	{
 		gchar *command;
@@ -1415,7 +1415,7 @@ on_build_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->project_root_dir)
 	{
-		build_configure_and_build (plugin, build_build_file_or_dir, plugin->project_root_dir);
+		build_configure_and_build (plugin, build_build_file_or_dir, plugin->project_root_dir, NULL, NULL, NULL);
 	}
 }
 
@@ -1424,7 +1424,7 @@ on_install_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->project_root_dir)
 	{
-		build_configure_and_build (plugin, build_install_dir,  plugin->project_root_dir);
+		build_configure_and_build (plugin, build_install_dir,  plugin->project_root_dir, NULL, NULL, NULL);
 	}
 }
 
@@ -1440,13 +1440,13 @@ on_clean_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 static void
 on_configure_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
-	build_configure_dialog (plugin, NULL, NULL);
+	build_configure_dialog (plugin, NULL, NULL, NULL, NULL, NULL);
 }
 
 static void
 on_build_tarball (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
-	build_configure_and_build (plugin, (BuildFunc) build_tarball, NULL);	
+	build_configure_and_build (plugin, (BuildFunc) build_tarball, NULL, NULL, NULL, NULL);	
 }
 
 static void
@@ -1459,7 +1459,7 @@ on_build_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	module = build_module_from_file (plugin, plugin->current_editor_file, NULL);
 	if (module != NULL)
 	{
-		build_configure_and_build (plugin, build_build_file_or_dir, module);
+		build_configure_and_build (plugin, build_build_file_or_dir, module, NULL, NULL, NULL);
 		g_object_unref (module);
 	}
 }
@@ -1469,7 +1469,7 @@ on_install_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	g_return_if_fail (plugin->current_editor_file != NULL);
 	
-	build_configure_and_build (plugin, build_install_dir,  plugin->current_editor_file);
+	build_configure_and_build (plugin, build_install_dir,  plugin->current_editor_file, NULL, NULL, NULL);
 }
 
 static void
@@ -1485,7 +1485,7 @@ on_compile_file (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	g_return_if_fail (plugin->current_editor_file != NULL);
 	
-	build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->current_editor_file);
+	build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->current_editor_file, NULL, NULL, NULL);
 }
 
 static void
@@ -1525,7 +1525,7 @@ fm_compile (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	g_return_if_fail (plugin->fm_current_file != NULL);
 
-	build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->fm_current_file);
+	build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->fm_current_file, NULL, NULL, NULL);
 }
 
 static void
@@ -1537,7 +1537,7 @@ fm_build (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	module = build_module_from_file (plugin, plugin->fm_current_file, NULL);
 	if (module != NULL)
 	{
-		build_configure_and_build (plugin, build_build_file_or_dir, module);
+		build_configure_and_build (plugin, build_build_file_or_dir, module, NULL, NULL, NULL);
 		g_object_unref (module);
 	}
 }
@@ -1547,7 +1547,7 @@ fm_install (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	g_return_if_fail (plugin->fm_current_file != NULL);
 	
-	build_configure_and_build (plugin, build_install_dir,  plugin->fm_current_file);
+	build_configure_and_build (plugin, build_install_dir,  plugin->fm_current_file, NULL, NULL, NULL);
 }
 
 static void
@@ -1564,7 +1564,7 @@ pm_compile (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	g_return_if_fail (plugin->pm_current_file != NULL);
 
-	build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->pm_current_file);
+	build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->pm_current_file, NULL, NULL, NULL);
 }
 
 static void
@@ -1577,7 +1577,7 @@ pm_build (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	module = build_module_from_file (plugin, plugin->pm_current_file, NULL);
 	if (module != NULL)
 	{
-		build_configure_and_build (plugin, build_build_file_or_dir, module);
+		build_configure_and_build (plugin, build_build_file_or_dir, module, NULL, NULL, NULL);
 		g_object_unref (module);
 	}
 }
@@ -1587,7 +1587,7 @@ pm_install (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	g_return_if_fail (plugin->pm_current_file != NULL);
 	
-	build_configure_and_build (plugin, build_install_dir,  plugin->pm_current_file);
+	build_configure_and_build (plugin, build_install_dir,  plugin->pm_current_file, NULL, NULL, NULL);
 }
 
 static void
@@ -1790,6 +1790,7 @@ update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 	DEBUG_PRINT ("%s", "Updating module UI");
 
 	has_file = bb_plugin->current_editor_file != NULL;
+	has_project = bb_plugin->project_root_dir != NULL;
 	if (has_file)
 	{
 		GFile *mod;
@@ -1799,7 +1800,7 @@ update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 		
 		mod = build_module_from_file (bb_plugin, bb_plugin->current_editor_file, &target);
 
-		if (!g_file_equal (mod, bb_plugin->project_root_dir) && !g_file_equal (mod, bb_plugin->project_build_dir))
+		if (has_project && !g_file_equal (mod, bb_plugin->project_root_dir) && !g_file_equal (mod, bb_plugin->project_build_dir))
 		{
 			module_name = g_file_get_basename (mod);
 			module = escape_label (module_name);
@@ -1820,7 +1821,6 @@ update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 			g_object_unref (mod);
 		}
 	}
-	has_project = bb_plugin->project_root_dir != NULL;
 
 	action = anjuta_ui_get_action (ui, "ActionGroupBuild",
 								   "ActionBuildBuildModule");
@@ -2067,12 +2067,24 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 	}
 	for (cfg = build_configuration_list_get_first (plugin->configurations); cfg != NULL; cfg = build_configuration_next (cfg))
 	{
-		gchar *key = g_strconcat("BuildArgs/", build_configuration_get_name (cfg), NULL);
+		gchar *key;
+		GList *list;
 
+		key = g_strconcat("BuildArgs/", build_configuration_get_name (cfg), NULL);
 		anjuta_session_set_string (session, "Build",
 			       key,
 			       build_configuration_get_args(cfg));
 		g_free (key);
+
+		list = build_configuration_get_variables (cfg);
+		if (list != NULL)
+		{
+			key = g_strconcat("BuildEnv/", build_configuration_get_name (cfg), NULL);
+			anjuta_session_set_string_list (session, "Build",
+				       key,
+			    	   list);
+			g_free (key);
+		}
 	}
 }
 
@@ -2100,15 +2112,35 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 
 	for (cfg = build_configuration_list_get_first (plugin->configurations); cfg != NULL; cfg = build_configuration_next (cfg))
 	{
-		gchar *key = g_strconcat("BuildArgs/", build_configuration_get_name (cfg), NULL);
-		gchar *args = anjuta_session_get_string (session, "Build",
-				key);
+		gchar *key;
+		gchar *args;
+		GList *env;
+		
+		key = g_strconcat("BuildArgs/", build_configuration_get_name (cfg), NULL);
+		args = anjuta_session_get_string (session, "Build", key);
 		g_free (key);
 		if (args != NULL)
 		{
 			build_configuration_set_args (cfg, args);
 			g_free (args);
 		}
+
+		key = g_strconcat("BuildEnv/", build_configuration_get_name (cfg), NULL);
+		env = anjuta_session_get_string_list (session, "Build",	key);
+		g_free (key);
+		if (env != NULL)
+		{
+			GList *item;
+
+			/* New variables are added at the beginning of the list */
+			for (item = env; item != NULL; item = g_list_next (item))
+			{
+				build_configuration_set_variable (cfg, (gchar *)item->data);
+				g_free (item->data);
+			}
+			g_list_free (env);
+		}
+
 	}
 
 	build_project_configured (G_OBJECT (plugin), NULL, NULL, NULL);
@@ -2658,7 +2690,7 @@ ibuildable_configure (IAnjutaBuildable *manager, const gchar *directory,
 	file = g_file_new_for_path (directory);
 	if (file == NULL) return;
 	
-	build_configure_dir (plugin, file, NULL, NULL, NULL);
+	build_configure_dir (plugin, file, NULL, NULL, NULL, NULL, NULL, NULL);
 	
 	g_object_unref (file);
 }
@@ -2673,7 +2705,7 @@ ibuildable_generate (IAnjutaBuildable *manager, const gchar *directory,
 	file = g_file_new_for_path (directory);
 	if (file == NULL) return;
 
-	build_generate_dir (plugin, file, NULL, NULL, NULL);
+	build_generate_dir (plugin, file, NULL, NULL, NULL, NULL, NULL, NULL);
 	
 	g_object_unref (file);
 }
@@ -2766,7 +2798,7 @@ ibuilder_build (IAnjutaBuilder *builder, const gchar *uri,
 	file = g_file_new_for_uri (uri);
 	if (file == NULL) return NULL;
 	
-	context = build_build_file_or_dir (plugin, file, callback, user_data, err);
+	context = build_configure_and_build (plugin, build_build_file_or_dir, plugin->project_root_dir, callback, user_data, NULL);
 	
 	g_object_unref (file);
 	
